@@ -7,12 +7,12 @@ from .event_selection import EventSelector, DL3FixedCuts, DataBinning
 from .io import (
     get_dataset_keys,
     auto_merge_h5files,
-    smart_merge_h5files,
     write_simtel_energy_histogram,
     write_mcheader,
     write_dl2_dataframe,
     global_metadata,
     add_global_metadata,
+    add_config_metadata,
     write_metadata,
     write_subarray_tables,
     read_simu_info_merged_hdf5,
@@ -20,6 +20,8 @@ from .io import (
     read_mc_dl2_to_QTable,
     read_data_dl2_to_QTable,
     HDF5_ZSTD_FILTERS,
+    get_srcdep_index_keys,
+    get_srcdep_params,
 )
 
 standard_config = get_standard_config()
@@ -34,12 +36,12 @@ __all__ = [
     'DataBinning',
     'get_dataset_keys',
     'auto_merge_h5files',
-    'smart_merge_h5files',
     'write_simtel_energy_histogram',
     'write_mcheader',
     'write_dl2_dataframe',
     'global_metadata',
     'add_global_metadata',
+    'add_config_metadata',
     'write_metadata',
     'write_subarray_tables',
     'read_simu_info_merged_hdf5',
@@ -47,5 +49,6 @@ __all__ = [
     'HDF5_ZSTD_FILTERS',
     'read_mc_dl2_to_QTable',
     'read_data_dl2_to_QTable',
-
+    'get_srcdep_index_keys',
+    'get_srcdep_params',
 ]
