@@ -1,27 +1,36 @@
-import h5py
-from multiprocessing import Pool
-import numpy as np
-import pandas as pd
-from astropy.table import Table, vstack, QTable
-import tables
-from tables import open_file
+import logging
 import os
+import re
+from multiprocessing import Pool
+
 import astropy.units as u
 import ctapipe
+import ctapipe_io_lst
+import h5py
 import lstchain
-from ctapipe.io import HDF5TableReader
+import numpy as np
+import pandas as pd
+import tables
+from astropy.table import Table, vstack, QTable
 from ctapipe.containers import SimulationConfigContainer
+# from ctapipe.tools.stage1 import Stage1ProcessorTool
+from ctapipe.instrument import (
+    OpticsDescription,
+    CameraGeometry,
+    CameraDescription,
+    CameraReadout,
+    TelescopeDescription,
+    SubarrayDescription
+)
+from ctapipe.io import HDF5TableReader
 from ctapipe.io import HDF5TableWriter
 from eventio import Histograms
 from eventio.search_utils import yield_toplevel_of_type
-from .lstcontainers import ThrownEventsHistogram, ExtraMCInfo, MetaData
-from tqdm import tqdm
-# from ctapipe.tools.stage1 import Stage1ProcessorTool
-from ctapipe.instrument import OpticsDescription, CameraGeometry, CameraDescription, CameraReadout, \
-    TelescopeDescription, SubarrayDescription
 from pyirf.simulations import SimulatedEventsInfo
+from tables import open_file
+from tqdm import tqdm
 
-import logging
+from .lstcontainers import ThrownEventsHistogram, ExtraMCInfo, MetaData
 
 log = logging.getLogger(__name__)
 
@@ -37,9 +46,9 @@ __all__ = [
     'check_metadata',
     'read_metadata',
     'auto_merge_h5files',
-    'smart_merge_h5files',
     'global_metadata',
     'add_global_metadata',
+    'add_config_metadata',
     'write_subarray_tables',
     'write_metadata',
     'write_dataframe',
@@ -49,16 +58,19 @@ __all__ = [
     'read_data_dl2_to_QTable',
     'read_dl2_params',
     'extract_observation_time',
-    'merge_dl2_runs'
+    'merge_dl2_runs',
+    'get_srcdep_index_keys',
+    'get_srcdep_params'
 ]
 
-dl1_params_tel_mon_ped_key = "dl1/event/telescope/monitoring/pedestal"
+dl1_params_tel_mon_ped_key = "/dl1/event/telescope/monitoring/pedestal"
 dl1_params_tel_mon_cal_key = "/dl1/event/telescope/monitoring/calibration"
-dl1_params_lstcam_key = "dl1/event/telescope/parameters/LST_LSTCam"
-dl1_images_lstcam_key = "dl1/event/telescope/image/LST_LSTCam"
-dl2_params_lstcam_key = "dl2/event/telescope/parameters/LST_LSTCam"
-dl1_params_src_dep_lstcam_key = "dl1/event/telescope/parameters_src_dependent/LST_LSTCam"
-dl2_params_src_dep_lstcam_key = "dl2/event/telescope/parameters_src_dependent/LST_LSTCam"
+dl1_params_tel_mon_flat_key = "/dl1/event/telescope/monitoring/flatfield"
+dl1_params_lstcam_key = "/dl1/event/telescope/parameters/LST_LSTCam"
+dl1_images_lstcam_key = "/dl1/event/telescope/image/LST_LSTCam"
+dl2_params_lstcam_key = "/dl2/event/telescope/parameters/LST_LSTCam"
+dl1_params_src_dep_lstcam_key = "/dl1/event/telescope/parameters_src_dependent/LST_LSTCam"
+dl2_params_src_dep_lstcam_key = "/dl2/event/telescope/parameters_src_dependent/LST_LSTCam"
 
 HDF5_ZSTD_FILTERS = tables.Filters(
     complevel=5,            # enable compression, 5 is a good tradeoff between compression and speed
@@ -72,6 +84,11 @@ def read_simu_info_hdf5(filename):
     """
     Read simu info from an hdf5 file
 
+    Parameters
+    ----------
+    filename: str
+        path to the HDF5 file
+
     Returns
     -------
     `ctapipe.containers.SimulationConfigContainer`
@@ -139,7 +156,7 @@ def get_dataset_keys(filename):
 
     def walk(name, obj):
         if type(obj) == h5py._hl.dataset.Dataset:
-            dataset_keys.append(name)
+            dataset_keys.append('/'+name)
 
     with h5py.File(filename, "r") as file:
         file.visititems(walk)
@@ -192,10 +209,18 @@ def stack_tables_h5files(filenames_list, output_filename="merged.h5", keys=None)
         merged_table.write(output_filename, path=k, append=True)
 
 
-def auto_merge_h5files(file_list, output_filename='merged.h5', nodes_keys=None, merge_arrays=False, filters=HDF5_ZSTD_FILTERS):
+def auto_merge_h5files(
+    file_list,
+    output_filename="merged.h5",
+    nodes_keys=None,
+    merge_arrays=False,
+    filters=HDF5_ZSTD_FILTERS,
+    progress_bar=True
+):
     """
     Automatic merge of HDF5 files.
     A list of nodes keys can be provided to merge only these nodes. If None, all nodes are merged.
+    It may be also used to create a new file output_filename from content stored in another file.
 
     Parameters
     ----------
@@ -204,39 +229,50 @@ def auto_merge_h5files(file_list, output_filename='merged.h5', nodes_keys=None,
     nodes_keys: list of path
     merge_arrays: bool
     filters
+    progress_bar : bool
+        Enabling the display of the progress bar during event processing.
     """
 
+    file_list = list(file_list)
+    if len(file_list) > 1:
+        file_list = merging_check(file_list)
+
     if nodes_keys is None:
         keys = set(get_dataset_keys(file_list[0]))
     else:
         keys = set(nodes_keys)
 
-    bar = tqdm(total=len(file_list))
+    bar = tqdm(total=len(file_list), disable=not progress_bar)
     with open_file(output_filename, 'w', filters=filters) as merge_file:
         with open_file(file_list[0]) as f1:
             for k in keys:
                 if type(f1.root[k]) == tables.table.Table:
-                    merge_file.create_table(
+                    t = merge_file.create_table(
                         os.path.join('/', k.rsplit('/', maxsplit=1)[0]),
                         os.path.basename(k),
                         createparents=True,
                         obj=f1.root[k].read()
                     )
+                    for att in f1.root[k].attrs._f_list():
+                        t.attrs[att] = f1.root[k].attrs[att]
                 if type(f1.root[k]) == tables.array.Array:
                     if merge_arrays:
-                        merge_file.create_earray(
+                        a = merge_file.create_earray(
                             os.path.join('/', k.rsplit('/', maxsplit=1)[0]),
                             os.path.basename(k),
                             createparents=True,
                             obj=f1.root[k].read()
                         )
                     else:
-                        merge_file.create_array(
+                        a = merge_file.create_array(
                             os.path.join('/', k.rsplit('/', maxsplit=1)[0]),
                             os.path.basename(k),
                             createparents=True,
                             obj=f1.root[k].read()
                         )
+                    for att in f1.root[k].attrs._f_list():
+                        a.attrs[att] = f1.root[k].attrs[att]
+
         bar.update(1)
         for filename in file_list[1:]:
             common_keys = keys.intersection(get_dataset_keys(filename))
@@ -251,9 +287,21 @@ def auto_merge_h5files(file_list, output_filename='merged.h5', nodes_keys=None,
                             # https://github.com/cta-observatory/cta-lstchain/issues/671
                             out_node.append(in_node.read().astype(out_node.dtype))
                     except:
-                        log.exception("Can't append node {} from file {}".format(k, filename))
+                        log.error("Can't append node {} from file {}".format(k, filename))
+                        raise
             bar.update(1)
 
+    # merge global metadata and store source file names
+    metadata0 = read_metadata(file_list[0])
+    source_filenames = [str(file_list[0])]
+    for file in file_list[1:]:
+        source_filenames.append(str(file))
+
+    with open_file(output_filename, mode="a") as file:
+        sources = file.create_group("/", "source_filenames", "List of files merged")
+        file.create_array(sources, "filenames", source_filenames, "List of files merged")
+    write_metadata(metadata0, output_filename)
+
 
 def merging_check(file_list):
     """
@@ -297,39 +345,12 @@ def merging_check(file_list):
             assert subarray_info == subarray_info0
 
         except AssertionError:
-            log.exception(f"{filename} cannot be smart merged '¯\_(ツ)_/¯'")
+            log.exception(f"{filename} cannot be merged '¯\_(ツ)_/¯'")
             mergeable_list.remove(filename)
 
     return mergeable_list
 
 
-def smart_merge_h5files(
-    file_list, output_filename="merged.h5", node_keys=None, merge_arrays=False
-):
-    """
-    Check that HDF5 files are compatible for merging and merge them
-
-    Parameters
-    ----------
-    file_list: list of paths to hdf5 files
-    output_filename: path to the merged file
-    node_keys
-    merge_arrays: bool
-    """
-    smart_list = merging_check(file_list)
-    auto_merge_h5files(
-        smart_list, output_filename, nodes_keys=node_keys, merge_arrays=merge_arrays
-    )
-
-    # Merge metadata
-    metadata0 = read_metadata(smart_list[0])
-    for file in smart_list[1:]:
-        metadata = read_metadata(file)
-        check_metadata(metadata0, metadata)
-        metadata0.SOURCE_FILENAMES.extend(metadata.SOURCE_FILENAMES)
-    write_metadata(metadata0, output_filename)
-
-
 def write_simtel_energy_histogram(source, output_filename, obs_id=None, filters=HDF5_ZSTD_FILTERS, metadata={}):
     """
     Write the energy histogram from a simtel source to a HDF5 file
@@ -764,7 +785,7 @@ def check_metadata(metadata1, metadata2):
         assert metadata1[k] == metadata2[k]
 
 
-def global_metadata(source):
+def global_metadata():
     """
     Get global metadata container
 
@@ -772,11 +793,12 @@ def global_metadata(source):
     -------
     `lstchain.io.lstcontainers.MetaData`
     """
+
     metadata = MetaData()
     metadata.LSTCHAIN_VERSION = lstchain.__version__
     metadata.CTAPIPE_VERSION = ctapipe.__version__
+    metadata.CTAPIPE_IO_LST_VERSION = ctapipe_io_lst.__version__
     metadata.CONTACT = "LST Consortium"
-    metadata.SOURCE_FILENAMES.append(os.path.basename(source.input_url))
 
     return metadata
 
@@ -795,6 +817,29 @@ def add_global_metadata(container, metadata):
         container.meta[k] = item
 
 
+def add_config_metadata(container, configuration):
+    """
+    Add configuration parameters to a container in container.meta.config
+
+    Parameters
+    ----------
+    container: `ctapipe.containers.Container`
+    configuration: config dict
+    """
+    linted_config = str(configuration)
+    linted_config = linted_config.replace("<LazyConfigValue {}>", "None")
+    linted_config = re.sub(r"<LazyConfigValue\svalue=(.*?)>", "\\1", linted_config)
+    linted_config = re.sub(r"DeferredConfigString\((.*?)\)", "\\1", linted_config)
+    linted_config = re.sub(r"PosixPath\((.*?)\)", "\\1", linted_config)
+    linted_config = linted_config.replace("\'", "\"")
+    linted_config = linted_config.replace("None", "\"None\"")
+    linted_config = linted_config.replace("inf",  "\"inf\"")
+    linted_config = linted_config.replace("True", "true")
+    linted_config = linted_config.replace("False", "false")
+
+    container.meta["config"] = linted_config
+
+
 def write_subarray_tables(writer, event, metadata=None):
     """
     Write subarray tables info to a HDF5 file
@@ -813,7 +858,7 @@ def write_subarray_tables(writer, event, metadata=None):
     writer.write(table_name="subarray/trigger", containers=[event.index, event.trigger])
 
 
-def write_dataframe(dataframe, outfile, table_path, mode="a", index=False):
+def write_dataframe(dataframe, outfile, table_path, mode="a", index=False, config=None, meta=None):
     """
     Write a pandas dataframe to a HDF5 file using pytables formatting.
 
@@ -823,6 +868,8 @@ def write_dataframe(dataframe, outfile, table_path, mode="a", index=False):
     outfile: path
     table_path: str
         path to the table to write in the HDF5 file
+    config: config metadata
+    meta: global metadata
     """
     if not table_path.startswith("/"):
         table_path = "/" + table_path
@@ -830,15 +877,20 @@ def write_dataframe(dataframe, outfile, table_path, mode="a", index=False):
     with tables.open_file(outfile, mode=mode) as f:
         path, table_name = table_path.rsplit("/", maxsplit=1)
 
-        f.create_table(
+        t = f.create_table(
             path,
             table_name,
             dataframe.to_records(index=index),
             createparents=True,
         )
+        if config:
+            t.attrs["config"] = config
+        if meta:
+            for k, item in meta.as_dict().items():
+                t.attrs[k] = item
 
 
-def write_dl2_dataframe(dataframe, outfile):
+def write_dl2_dataframe(dataframe, outfile, config=None, meta=None):
     """
     Write DL2 dataframe to a HDF5 file
 
@@ -846,8 +898,10 @@ def write_dl2_dataframe(dataframe, outfile):
     ----------
     dataframe: `pandas.DataFrame`
     outfile: path
+    config: config metadata
+    meta: global metadata
     """
-    write_dataframe(dataframe, outfile=outfile, table_path=dl2_params_lstcam_key)
+    write_dataframe(dataframe, outfile=outfile, table_path=dl2_params_lstcam_key, config=config, meta=meta)
 
 
 def add_column_table(table, ColClass, col_label, values):
@@ -1120,3 +1174,58 @@ def merge_dl2_runs(data_tag, runs, columns_to_read=None, n_process=4):
     observation_time = sum([t.total_seconds() for t in observation_times])
     df = pd.concat(df_list)
     return observation_time, df
+
+
+def get_srcdep_index_keys(filename):
+    """
+    get index column name of source-dependent multi index columns
+
+    Parameters
+    ----------
+    filename: str - path to the HDF5 file
+
+    Returns
+    -------
+    source-dependent index names
+    """
+    dataset_keys = get_dataset_keys(filename)
+
+    if dl2_params_src_dep_lstcam_key in dataset_keys:
+        data =  pd.read_hdf(filename, key=dl2_params_src_dep_lstcam_key)
+
+    elif dl1_params_src_dep_lstcam_key in dataset_keys:
+        data =  pd.read_hdf(filename, key=dl1_params_src_dep_lstcam_key)
+
+    if not isinstance(data.columns, pd.MultiIndex):
+        data.columns = pd.MultiIndex.from_tuples(
+            [tuple(col[1:-1].replace('\'', '').replace(' ', '').split(",")) for col in data.columns])
+
+    return data.columns.levels[0]
+
+
+def get_srcdep_params(filename, key):
+    """
+    get srcdep parameter data frame
+
+    Parameters
+    ----------
+    filename: str - path to the HDF5 file
+    key: `str` - multi index key corresponding to an expected source position (e.g. 'on', 'off_180')
+
+    Returns
+    -------
+    `pandas.DataFrame`
+    """
+    dataset_keys = get_dataset_keys(filename)
+
+    if dl2_params_src_dep_lstcam_key in dataset_keys:
+        data =  pd.read_hdf(filename, key=dl2_params_src_dep_lstcam_key)
+
+    elif dl1_params_src_dep_lstcam_key in dataset_keys:
+        data =  pd.read_hdf(filename, key=dl1_params_src_dep_lstcam_key)
+
+    if not isinstance(data.columns, pd.MultiIndex):
+        data.columns = pd.MultiIndex.from_tuples(
+            [tuple(col[1:-1].replace('\'', '').replace(' ', '').split(",")) for col in data.columns])
+        
+    return data[key]
