@@ -19,7 +19,7 @@ from ctapipe.image import (
     hillas_parameters,
     tailcuts_clean,
 )
-from ctapipe.image.morphology import number_of_islands
+from ctapipe.image import number_of_islands, apply_time_delta_cleaning
 from ctapipe.io import EventSource, HDF5TableWriter
 from ctapipe.utils import get_dataset_path
 from traitlets.config import Config
@@ -28,11 +28,11 @@ from ctapipe.containers import EventType
 from . import disp
 from .utils import sky_to_camera
 from .volume_reducer import apply_volume_reduction
-from ..calib.camera import lst_calibration, load_calibrator_from_config
+from ..calib.camera import load_calibrator_from_config
 from ..calib.camera.calibration_calculator import CalibrationCalculator
 from ..image.muon import analyze_muon_event, tag_pix_thr
 from ..image.muon import create_muon_table, fill_muon_event
-from ..image.cleaning import apply_time_delta_cleaning, apply_dynamic_cleaning
+from ..image.cleaning import apply_dynamic_cleaning
 from ..io import (
     DL1ParametersContainer,
     replace_config,
@@ -41,6 +41,7 @@ from ..io import (
 )
 from ..io import (
     add_global_metadata,
+    add_config_metadata,
     global_metadata,
     write_calibration_data,
     write_mcheader,
@@ -84,7 +85,6 @@ def setup_writer(writer, subarray, is_simulation):
 
     for tel_name in tel_names:
         # None values in telescope/image table
-        writer.exclude(f'telescope/image/{tel_name}', 'image_mask')
         writer.exclude(f'telescope/image/{tel_name}', 'parameters')
         # None values in telescope/parameters table
         writer.exclude(f'telescope/parameters/{tel_name}', 'hadroness')
@@ -101,6 +101,7 @@ def setup_writer(writer, subarray, is_simulation):
             writer.exclude(f'telescope/parameters/{tel_name}', 'dragon_time')
             writer.exclude(f'telescope/parameters/{tel_name}', 'ucts_time')
             writer.exclude(f'telescope/parameters/{tel_name}', 'tib_time')
+            writer.exclude(f'telescope/parameters/{tel_name}', 'ucts_jump')
             writer.exclude(f'telescope/parameters/{tel_name}', 'ucts_trigger_type')
         else:
             writer.exclude(f'telescope/parameters/{tel_name}', 'mc_energy')
@@ -168,14 +169,6 @@ def get_dl1(
     n_pixels = np.count_nonzero(signal_pixels)
 
     if n_pixels > 0:
-        # check the number of islands
-        num_islands, island_labels = number_of_islands(camera_geometry, signal_pixels)
-
-        if use_main_island:
-            n_pixels_on_island = np.bincount(island_labels)
-            n_pixels_on_island[0] = 0  # first island is no-island and should not be considered
-            max_island_label = np.argmax(n_pixels_on_island)
-            signal_pixels[island_labels != max_island_label] = False
 
         if delta_time is not None:
             signal_pixels = apply_time_delta_cleaning(
@@ -188,13 +181,21 @@ def get_dl1(
 
         if use_dynamic_cleaning:
             threshold_dynamic = config['dynamic_cleaning']['threshold']
-            fraction_dynamic = config['dynamic_cleaning'][
-                'fraction_cleaning_intensity']
+            fraction_dynamic = config['dynamic_cleaning']['fraction_cleaning_intensity']
             signal_pixels = apply_dynamic_cleaning(image,
                                                    signal_pixels,
                                                    threshold_dynamic,
                                                    fraction_dynamic)
 
+        # check the number of islands
+        num_islands, island_labels = number_of_islands(camera_geometry, signal_pixels)
+
+        if use_main_island:
+            n_pixels_on_island = np.bincount(island_labels)
+            n_pixels_on_island[0] = 0  # first island is no-island and should not be considered
+            max_island_label = np.argmax(n_pixels_on_island)
+            signal_pixels[island_labels != max_island_label] = False
+
         # count surviving pixels
         n_pixels = np.count_nonzero(signal_pixels)
 
@@ -230,8 +231,12 @@ def get_dl1(
     # image:
     dl1_container.set_telescope_info(subarray, telescope_id)
 
+    # save the applied cleaning mask
+    calibrated_event.dl1.tel[telescope_id].image_mask = signal_pixels
+
     return dl1_container
 
+
 def r0_to_dl1(
     input_filename=get_dataset_path('gamma_test_large.simtel.gz'),
     output_filename=None,
@@ -266,14 +271,12 @@ def r0_to_dl1(
 
     config = replace_config(standard_config, custom_config)
 
-    custom_calibration = config["custom_calibration"]
-
     source = EventSource(input_url=input_filename,
                          config=Config(config["source_config"]))
     subarray = source.subarray
     is_simu = source.is_simulation
 
-    metadata = global_metadata(source)
+    metadata = global_metadata()
     write_metadata(metadata, output_filename)
 
     cal_mc = load_calibrator_from_config(config, subarray)
@@ -354,8 +357,8 @@ def r0_to_dl1(
             # write sub tables
             if is_simu:
                 write_subarray_tables(writer, event, metadata)
-                if not custom_calibration:
-                    cal_mc(event)
+                cal_mc(event)
+
                 if config['mc_image_scaling_factor'] != 1:
                     rescale_dl1_charge(event, config['mc_image_scaling_factor'])
 
@@ -370,6 +373,9 @@ def r0_to_dl1(
 
                     #initialize the event monitoring data
                     event.mon = deepcopy(source.r0_r1_calibrator.mon_data)
+                    for container in [event.mon.tel[tel_id].pedestal, event.mon.tel[tel_id].flatfield, event.mon.tel[tel_id].calibration]:
+                        add_global_metadata(container, metadata)
+                        add_config_metadata(container, config)
 
                     # write the first calibration event (initialized from calibration h5 file)
                     write_calibration_data(writer,
@@ -412,12 +418,8 @@ def r0_to_dl1(
                 # extra info for the image table
                 extra_im.tel_id = telescope_id
                 extra_im.selected_gain_channel = event.r1.tel[telescope_id].selected_gain_channel
-
-                # write image first, so we are sure nothing here modifies it
-                writer.write(
-                    table_name=f'telescope/image/{tel_name}',
-                    containers=[event.index, dl1_tel, extra_im]
-                )
+                add_global_metadata(extra_im, metadata)
+                add_config_metadata(extra_im, config)
 
                 focal_length = subarray.tel[telescope_id].optics.equivalent_focal_length
                 mirror_area = subarray.tel[telescope_id].optics.mirror_area
@@ -430,9 +432,6 @@ def r0_to_dl1(
                 dl1_container.fill_event_info(event)
 
 
-                if custom_calibration:
-                    lst_calibration(event, telescope_id)
-
                 # Will determine whether this event has to be written to the
                 # DL1 output or not.
                 if is_simu:
@@ -461,7 +460,9 @@ def r0_to_dl1(
                     # FIXME: just keep it as time, table writer and reader handle it
                     dl1_container.dragon_time = event.trigger.time.unix
                     dl1_container.tib_time = 0
-
+                    if 'ucts_jump' in vars(event.lst.tel[
+                                               telescope_id].evt.__class__):
+                        dl1_container.ucts_jump = event.lst.tel[telescope_id].evt.ucts_jump
                     dl1_container.ucts_trigger_type = event.lst.tel[telescope_id].evt.ucts_trigger_type
                     dl1_container.trigger_type = event.lst.tel[telescope_id].evt.tib_masked_trigger
                 else:
@@ -478,10 +479,16 @@ def r0_to_dl1(
 
                 for container in [extra_im, dl1_container, event.r0, dl1_tel]:
                     add_global_metadata(container, metadata)
+                    add_config_metadata(container, config)
 
                 writer.write(table_name=f'telescope/parameters/{tel_name}',
                              containers=[event.index, dl1_container])
 
+                writer.write(
+                    table_name=f'telescope/image/{tel_name}',
+                    containers=[event.index, dl1_tel, extra_im]
+                )
+
                 # Muon ring analysis, for real data only (MC is done starting from DL1 files)
                 if not is_simu:
                     bad_pixels = event.mon.tel[telescope_id].calibration.unusable_pixels[0]
